

 
%=========================================================================%
% [A]. DECLARATION OF VARIABLES 
%=========================================================================%

%-------------------------------------------------------------------------%
% [1]. Endogenous Variables                                               %
%-------------------------------------------------------------------------%
var PX_d    //domestic inflation
    PXstar  //foreign inflation
    C       //the aggregate consumption of households
    N       //hours worked W       //wage 
    
    R       //domestic interest rates
    Rstar   //foreign interest rates
    NX      //net exports
    fxr_star //foreign exchange reserve
    Ko      //Oil capital
    Kno     //Non-oil Capital 
    KG      //public capital 
    RER     //Real exchange rates
    b       //fiscal debt
    bstar   //foreign debt
    T       //Lump-sum taxes
    GC      //Public Consumption
    GI      //Public investment
    I       // private investment
    Y       //aggregate output
    Yo      //Oil output
    Yno    // Non-oil output
    Ystar   //foreign output
    p_h     //domestic prices
    p_g     //Government purchase
    Pstar_o //The world oil prices
    SWF     //Sovereign Wealth fund assets
    FDI     //Foreign Direct Investment
    
    OR      //Oil Revenue
    //PX_o    //Oil-price inflation
    //PX_h    //Domestic inflation
    Z       //Technology shock
    //W
    
    eps_r   //shock to interest rates
    eps_fx  //shock to foreign exchange
 
//yobs    //Observed (domestic) output
//cobs    //Observed (domestic consumption
;

%-------------------------------------------------------------------------%
% [2]. Exogenous Variables                                                %
%-------------------------------------------------------------------------% 
varexo u_Z u_r e_o u_fx; //

predetermined_variables RER;

%-------------------------------------------------------------------------%
% [3]. Parameters  (including steady state)                                                       %
%-------------------------------------------------------------------------%

parameters betta        //domestic discount factor
           betta_star   //foreign discount rate 
           THETTA_n
           THETTA_i 
           muo          //the fraction of rule-of-thumb consumers in the economy
           kappax        //investment adjustment cost parameter
           delta        //depreciation rate of private capital
           delta_g      //depreciation rate of public capital
           OMEGA        //the upper bound of leverage ratio
           phi          //wage elasticity to hours worked
           phi_px       //inflation response in the Taylor rule
           phi_y        //output-gap in the Taylor rule 
           phi_b        //
           phi_I        //
           phi_C        //
           phi_OR       //
           phiStar      //
           phiStar_px   //
           phiStar_y    //
           sigmae       //the inverse of the intertemporal elasticity of subsitution
           sigmae_o     //standard deviation of the world oil price
           rho          //interest rate smoothing in the Taylor rule
           rho_r        //Persistence of intrerest rate shocks
           rho_swf      //persistence in SWF process
           rho_fx       //
           rho_fxr      //persistence in foreign exchange reserve of the central bank
           rho_fdi      //FDI response to the world oil-price
           rho_o        //persistence in the world oil-price process
           rho_GI       //
           rho_GC       //
           rho_ystar    //
           rho_Z        //persistence of productivity shock
           lota_div     //the dividend share of oil profit accrued to government     
           tau_o        //oil royalty rate
           alppha       //non-oil out put elasticity to private capital
           alpa1        //exchange rate response in the intervention rule
           alpa2        //exchange rate change response in the intervetion rule
           alpa_o       //oil output elasticity to private capital
           alpa_no      //non-oil output elasticity to private capital
           alpaStar     //
           PSI          //non-oil output elasticity to public capital.
           gammaa       //home-bias in private consumption and investment
           gammaa2      //home-bias in government purchase
           gammaa_GI    //response of the public Investment to fiscal debt
           gammaa_GC    //response of the public consumption to fiscal debt
           gammaa_IOF   //response of public invesment to oil revenue
           gammaa_COF   //response of public consumption to oil revenue
           
           zeta_GI      //response of public investment to output
           zeta_GC      //response of public consumption to output
           epi          //elasticity of subsitution beteween different intermediate goods
           theta        //index of domestic price stickness
           thetaStar   //index of foreign price stickness           
    
           nx_y         //ratio of net export to output
           bstar_y      //ratio of foreign bond to output
           swf_y        //ratio of SWF to output
           fdi_y        //ratio of FDI to output
           s_o          //
           gI_y         //ratio of public investment to output
           gC_y         //ratio of public consumption to output
           b_y          //ratio of bond to output
           c_y          //ratio of consumption to output
           i_y          //ratio of investment to output
           e_ystar      //
     
           T_SS         //Steady-state aggregate taxes
           TS_SS        //Steady-state taxes on optimizers 
           Y_SS         //Steady-state aggregate output
           Yo_SS        //Steady-state oil output
           Yno_SS       //Steady-state non-oil output
           Kno_SS       //Steady-state capital of non-oil sector
           Ko_SS        //Steady-state capital of oil sector
           b_SS         //Steady-state domestic fiscal bond (debt) 
           bstar_SS     //Steady-state RoW fiscal bond (debt) 
           C_SS         //Steady-state consumption
           //CS_SS        //Steady-state consumption of Optimizers
           //CN_SS        //Steady-state consumption of Rule-of-thumb
           N_SS         //Steady-state labour supply
           NX_SS        //Steady-state net exports
           R_SS         //Steady-state domestic nominal interest rate
           Rstar_SS     //Steady-state RoW nominal interest rate
           GI_SS        //Steady-state government (public) investment
           GC_SS        //Steady-state government (public) consumption
           KG_SS        //Steady-state public capital accumulation
           RER_SS       //Steady-state RER
           OR_SS        //Steady-state oil revenue
           I_SS         //Steady-state investment
           fxr_SS       //Steady-state foreign exchange reserve
           SWF_SS       //Steady-state SWF
           FDI_SS       //Steady-state FDI
           W_SS         //Steady-state wage
           Q_SS         //Steady-state Tobin Q (shadow price of capital)
           lambda       //Price setting condition - Lagrange multiplier
           lambda_star; //Foreign multiplier
            
%-------------------------------------------------------------------------%
% [4].Parameters Values                                                   %
%-------------------------------------------------------------------------%
            
           betta=0.978;        //domestic discount factor (6% rate of return)
           betta_star=0.9901;   //foreign discount rate (1% rate of return) 
           muo=0.5;             //the fraction of rule-of-thumb consumers in the economy
           kappax=20.0;        //investment adjustment cost parameter 20.0;
           delta=0.025;        //depreciation rate of private capital
           delta_g=0.02;      //depreciation rate of public capital
           OMEGA=0.54;        //the upper bound of leverage ratio 0.54 
           phi=1.45;          //wage elasticity to hours worked
           phi_px=1.7;       //inflation response in the Taylor rule
           phi_y=0.3;        //output-gap in the Taylor rule    
           phi_b=0.40;       //elasticity of fiscal bonds (debt)
           phi_I=0.20;        //elasticity of government investment
           phi_C=1.00;        //elasticity of government consumption
           phi_OR=-0.3;       //elasticity of Oil Revenue
           phiStar=1.45;      //
           phiStar_px=1.5;   //
           phiStar_y=0.125;    //
           sigmae=2.00;       //the inverse of the intertemporal elasticity of subsitution
           sigmae_o=0.15;     //standard deviation of the world oil price
           rho=0.95;          //interest rate smoothing in the Taylor rule
           rho_r=0.65;          //interest rate smoothing in the Taylor rule      
           rho_swf=0.775;      //persistence in SWF process
           rho_fx=0.4;       //*****************Assigned value needs for empirical value.
           rho_fxr=0.7;      //persistence in foreign exchange reserve of the central bank
           rho_fdi=0.20;   //1-rho_fdi=0.80;/      //FDI response to the world oil-price
           rho_o=0.80;        //persistence in the world oil-price process =0.98
           rho_GI=0.00;       //
           rho_GC=0.53;       //
           rho_ystar=0.80;    //
           rho_Z=0.90;        //persistence of productivity shock
           lota_div=0.05;     //the dividend share of oil profit accrued to government     
           tau_o=0.33;        //oil royalty rate
           alpa =0.3;         //non-oil out put elasticity to private capital
           
           alpa1=-0.18;        //exchange rate response in the intervention rule
           alpa2=-1.57;        //exchange rate change response in the intervention rule
           alpa_o=0.7;       //oil output elasticity to private capital
           alpa_no=0.3;      //non-oil output elasticity to private capital
           alpaStar=0.32;    //
           PSI=0.16 ;          //non-oil output elasticity to public capital.
           gammaa =0.68;       //home-bias in private consumption and investment
           gammaa2=0.9;      //home-bias in government purchase
                
           gammaa_GI=0.30;    //response of the public Investment to fiscal debt
           gammaa_GC=0.30;    //response of the public consumption to fiscal debt
           gammaa_IOF=0.10;   //response of public invesment to oil revenue
           gammaa_COF=0.20;   //response of public consumption to oil revenue
           zeta_GI=-0.54;     //response of public investment to output //0.54 positive value for pro-cyclical and negative for counter-cyclical fiscal policy;
           zeta_GC=-0.30;      //response of public consumption to output //positive for the pro-cyclical but negative for the counter-cyclical policy.
           epi=9.00;         //elasticity of subsitution between different intermediate goods
           theta=0.9;        //index of domestic price stickness 0.9
           thetaStar=0.75;   //index of foreign price stickness 0.5
           %eps_r =0.6;        //************Assigned need for empirical values
           %eps_fx=0.4;       //************Assigned need for empirical values
           
           nx_y =0.2;        //ratio of net export to output
           bstar_y=0.05;      //ratio of foreign bond to output
           swf_y=0.30;        //ratio of SWF to output
           fdi_y=0.05;        //ratio of FDI to output
           s_o=0.4;          //share of oil in total output (i.e. 40%).
           gI_y=0.35;         //ratio of public investment to output
           gC_y=0.18;        //ratio of public consumption to output
           b_y=0.018;          //ratio of bond to output
           c_y=0.60;          //ratio of consumption to output
           i_y=0.40;          //ratio of investment to output
           e_ystar=0.01;      //
           lambda_star=betta_star; //equating foreign multipllier to the discount rate



%---------------------------------------------------------------------------
R_SS=1/betta;
Rstar_SS=1/betta_star;
fxr_SS=betta_star-betta;
T_SS=(((b_SS^(phi_b))*(GI_SS^(phi_I))*(GC_SS^(phi_C)))/(OR_SS^phi_OR));
//C_SS=muo*CN_SS+(1-muo)*CS_SS;
N_SS=(W_SS)^(1/(phi-1));

%-------------------------------------------------------------------------%
% [4].Parameters estimation                                               %
%-------------------------------------------------------------------------%
    THETTA_n=(muo*(N_SS^phi)*phi+(1-muo)*(N_SS^phi))*(C_SS^-1);
    //THETTA_n=0.03;
    THETTA_i=((sigmae*C_SS)^-1)*(1-muo)*(C_SS-((phi^-1)*(N_SS^phi)));
    //THETTA_i=0.05;
    i_y=c_y+gC_y+gI_y+nx_y;
    lambda=((1-betta*theta)*(1-theta))/theta;
    
 
%=========================================================================%
% [B].MODEL                                                               %
%=========================================================================%



model(linear);

//[1]. Consumption Euler Equation [Equation 41 ******] 
        C=C(+1)+THETTA_n*(N-N(+1))-THETTA_i*(R-PX_d(+1))+muo*((T_SS*C_SS)^-1)*(T(+1)-T);
        
//[2]. FOC w.r.t. Non-oil capital and Investment [Equation 42 ******] 
        (kappax*(1+betta))*I=((betta*(1-delta)+OMEGA*fxr_SS)*(kappax*(1+betta)*I(+1)-kappax*betta*I(+2)-kappax*I)+kappax*betta*I(+1)
        +(1-betta*(1-delta)-OMEGA*fxr_SS)*(phi*N(+1)-Kno)-((C_SS-((N_SS^phi)/phi))/(sigmae*C_SS))*(R-PX_d(+1))
        +kappax*I(-1)+OMEGA*fxr_SS*(PXstar(+1)+RER-RER(+1)+fxr_star));  
        
//[3]. Log-linearized BOP Equations [Equation 43 ******] 
        NX=(((Rstar_SS*bstar_y*(1-muo))/nx_y)-(((Rstar_SS-rho_swf)*swf_y)/nx_y)-(fdi_y/nx_y)+(((1-lota_div)*((1-tau_o))*s_o)/nx_y))*RER
        -(((bstar_y*(1-muo))/nx_y)*bstar)+(((Rstar_SS*bstar_y*(1-muo))/nx_y)*bstar(-1))+(((Rstar_SS*swf_y-(1-muo)*Rstar_SS*bstar_y)/nx_y)*PXstar)-((fdi_y/nx_y)*FDI)
        -(((Rstar_SS*bstar_y*(1-muo))/nx_y)*RER(-1))+((((1-lota_div)*((1-tau_o)*s_o))/nx_y)*Yo)-((((Rstar_SS-rho_swf)*swf_y)/nx_y)*SWF(-1))
        +((((1-muo)*bstar_y-swf_y)/nx_y)*Rstar(-1))+(((((1-lota_div)*(1-tau_o))*s_o)/nx_y)*Pstar_o);

//[4]. The collateral constraints [Equation 44 ******] 
        bstar=PXstar(+1)-Rstar+Kno+RER-RER(+1)+kappax*(1+betta)*I(+1)-kappax*betta*I(+2)-kappax*I;

//[5]. Law of motion for non-oil capital [Equation 45 ******]
        Kno=(1-delta)*Kno(-1)+delta*I;

//[6]. Public Capital accumulation [Equation 46 ******]
        KG=(1-delta_g)*KG(-1)+delta_g*GI;

//[7]. The Oil Capital is accumulated by FDI [Equation 47 ******]
        Ko=(1-delta)*Ko(-1)+delta*FDI;

//[8]. Sovereign Wealth Fund (SWF) [Equation 48 ******]
        SWF=rho_swf*SWF(-1)+(((tau_o+lota_div*(1-tau_o))*s_o)/swf_y)*(Yo+Pstar_o);

//[9]. UIP Condition [Equation 49 ******]
        RER(+1)=R+RER-((betta_star/betta)*Rstar)+PXstar(+1)-PX_d(+1)-((betta_star/betta)-1)*fxr_star;
        //R=PX_d(+1)+((betta_star/betta)*Rstar)-PXstar(+1)+RER(+1)-RER+(((betta_star/betta)-1)*fxr_star);
         //RER=R(-1)+RER(-1)-((betta_star/betta)*Rstar(-1))+PXstar-PX_d-((betta_star/betta)-1)*fxr_star(-1);


//[10]. The Non-Oil production function [Equation 50 ******]
        Yno=Z+alppha*Kno(-1)+(1-alppha)*N+PSI*KG(-1);

//[11]. The Oil production function [Equation 51 ******]
        Yo=alpa_o*Ko(-1);

//[12]. The aggregate output  [Equation 52 ******]
        Y=(1-s_o)*Yno+(1-s_o)*p_h+s_o*(Yo+RER+Pstar_o);

//[13]. The government budget constraint [Equation 53 ******]
        b=R_SS*(b(-1)+R(-1)-PX_d)+(gI_y/((1-muo)*b_y))*GI+(gC_y/((1-muo)*b_y))*GC+((gC_y+gI_y)/((1-muo)*b_y))*p_g
        -(T_SS/(b_SS*(1-muo)))*T-(((Rstar_SS-rho_swf)*swf_y)/(b_y*(1-muo)))*OR;

//[14].Where Oil Revenue follows: 
            OR=RER+SWF(-1)+(1/(Rstar_SS-rho_swf))*Rstar(-1)-(Rstar_SS/(Rstar_SS-rho_swf))*PXstar;

//[15]. The relative price of Government purchases [Equation 54 ******]
        p_g=gammaa2*p_h+(1-gammaa2)*RER;

//[16]. The domestic Goods Market Clearing condition [Equation 55 ******] 
        Yno+p_h=((gammaa*c_y)/(1-s_o))*C+(((1-muo)*(1-i_y)*gammaa)/(1-s_o))*I+((gammaa2*gC_y)/(1-s_o))*GC
        +((gammaa2*gI_y)/(1-s_o))*GI+((gammaa2*(gC_y+gI_y)/(1-s_o))*p_g);

//[17]. The real GDP  in terms of Investment [Equation 56 ******]
        I=(1/((1-i_y)*(1-muo)))*(Y-(c_y*C)-(gC_y*GC)-(gI_y*GI)-((gC_y+gI_y)*p_g)-(nx_y*NX)); 

//[18]. The Philips curve [Equation 40 ******] 
        PX_d=betta*PX_d(+1)+((1-gammaa)/gammaa)*(lambda*((1-alppha)/(1-alppha+alppha*epi))+betta+1)*RER -((1-gammaa)/gammaa)*RER(-1)
        -betta*((1-gammaa)/gammaa)*RER(+1)+lambda*((1-alppha)/(1-alppha+alppha*epi))*(phi*N-Yno);
//[00].where
          //lambda=((1-betta*theta)*(1-theta))/theta; //definition of lambda

//    R=rho*R(-1)+(1-rho)*(phi_px*PX_d+phi_y*Y)+eps_r;


      R=rho*R(-1)+(1-rho)*(phi_px*(s_o*(((Pstar_o-Pstar_o(-1)))+(RER-RER(-1)))+ (1-s_o)*(PX_d-((1-gammaa)/gammaa)*(RER-RER(-1))))+phi_y*Y)+eps_r;


//[19]. Monetary Policy (Taylor (1993)) Rule [Equation 25 ******]
        //R=rho*R(-1)+(1-rho)*(phi_px*PX_d+phi_y*Y)+eps_r;                         //+eps_r;

//[20]. Exchange rate Policy Rule [Equation 26 ******] 
        //R=rho*R(-1)+(1-rho)*(phi_px*(s_o*PX_o+ (1-s_o)*PX_h)+phi_y*Y)+eps_fx;   //+eps_fx;
        
//[21].where
            //PX_o=((Pstar_o-Pstar_o(-1))/Pstar_o(-1))+((RER-RER(-1))/RER(-1));
            //PX_o=((Pstar_o-Pstar_o(-1)))+(RER-RER(-1)-PXstar+PX_d);
            //PX_o=((Pstar_o-Pstar_o(-1)))+(RER-RER(-1));
//[22].where
            //PX_h=PX_d - ((1-gammaa)/gammaa)*((RER-RER(-1))/RER(-1));
            //PX_h=PX_d - ((1-gammaa)/gammaa)*(RER-RER(-1)-PXstar+PX_d);
            //PX_h=PX_d - ((1-gammaa)/gammaa)*(RER-RER(-1));

//[23]. The Central Bank's Intervention or response to Exchange rate Depreciation rule [Equation 27 ****]
        fxr_star=rho_fxr*fxr_star(-1)+(1-rho_fxr)*(alpa1*RER+alpa2*(RER-RER(-1)-PXstar+PX_d));
        
//[24]. FDI Accumulation Process [Equation 14 *****]
        FDI=rho_fdi*FDI(-1)+(1-rho_fdi)*Pstar_o;

//[25]. World Oil price [Equation 15 *****]
        Pstar_o=rho_o*Pstar_o(-1)-e_o;

//[26]. Public Investment [Equation 22 *****]
        GI=rho_GI*GI(-1)+(1-rho_GI)*(zeta_GI*Yo -gammaa_GI*b(-1)+gammaa_IOF*OR);

//[27]. Public Consumption [Equation 23 *****]
        GC=rho_GC*GC(-1)+(1-rho_GC)*(zeta_GC*Yo -gammaa_GC*b(-1)+gammaa_COF*OR);

//[28]. Lump-sum Taxes      [Equation 24 *****]
        T=phi_b*b(-1)+phi_I*GI+phi_C*GC-phi_OR*OR;

//[29]. The rest of the World [Equation 30 -**** Foreign Output]
        Ystar=rho_ystar*Ystar(-1);      //+e_ystar;

//[30]. The rest of the World [Equation 31 -*** Foreign nominal interest rates]
        Rstar=phiStar_px*PXstar+phiStar_y*Ystar; 

//[31]. Labour supply condition [Equation 38 -*** Has to be log-linearized first]
        //W=C+(phi-1)*N;
        //W = (phi-1)*N;
        //C+T=W+N;

//[32]. The rest of the World [Equation 32 -**** Foreign Inflation]
        PXstar=betta_star*PXstar(+1)+lambda_star*(sigmae+((phiStar+alpaStar)/(1-alpaStar)))*Ystar;

//[A]. Evolution of Exogenous Processes
        eps_r = rho_r*eps_r(-1)+ u_r; //+ u_r;           // [33].eps_R %% eps_r=rho*eps_r(-1);            //+e_r; //[33]. evolution of exogenous interest rate processes
        Z = rho_Z*Z(-1) + u_Z;                  //[34]. evolution of exogenous productivity processes
        eps_fx=rho_fx*eps_fx(-1)+u_fx;         //[35]. Exogenous processes for foreign exchange.
        
//[35-36]. Observed variable relationship with the Model variables 
        //yobs = Y -steady_state(Y)+ME;
        //cobs = C -steady_state(C)+ME;


end;

//Observed variables
//varobs yobs cobs;

%-------------------------------------------------------------------------%
%%STEADY STATE = Computed from the Steadystate file.                                                         %%
%-------------------------------------------------------------------------%

%=========================================================================%
%%Initial Values (@all=0, since the model is linear)
%=========================================================================%  
initval;
    PX_d=0;      //domestic inflation
    PXstar=0;  //foreign inflation
    C=0;       //the aggregate consumption of households
    N=0;       //hours worked W=0;        //wage
    R=0;       //domestic interest rates
    Rstar=0;   //foreign interest rates
    NX=0;      //net exports
    fxr_star=0; //foreign exchange reserve
    Ko=0;      //Oil capital
    Kno=0;     //Non-oil Capital 
    KG=0;      //public capital 
    RER=0;     //Real exchange rates
    b=0;       //fiscal debt
    bstar=0;   //foreign debt
    T=0;       //Lump-sum taxes
    GC=0;      //Public Consumption
    GI=0;      //Public investment
    I=0;       // private investment
    Y=0;       //aggregate output (put GDP=1 for the steady state to balance)
    Yo=0;     //Oil output
    Yno=0;    // Non-oil output
    Ystar=0;   //foreign output
    p_h=0;     // domestic prices
    p_g=0;     //Government purchase
    Pstar_o=0; //The world oil prices
    SWF=0;     //Sovereign Wealth fund assets
    FDI=0;     //Foreign Direct Investment
    
    OR=0;      //Oil Revenue
    //PX_o=0;    //Oil-price inflation
    //PX_h=0;    //Domestic inflation
    Z=0;     //Technology shock
    //W = 0;
    //yobs=0;    //Observed (domestic) output
    //cobs=0;    //Observed (domestic consumption
   
    eps_r = 0; //Interest rates shocks
    eps_fx = 0; //Interest rates shocks  
end;

%%========================================================================%
%%SHOCK PROCESSES
%=========================================================================%
shocks;
    //var u_Z; stderr 0.01; //productivity shock
    //var u_r; stderr 0.01; //interest rate shock
    //var u_fx; stderr 0.01; //foreign exchange shock
    var e_o; stderr 0.01; //international oil-price shock
   //var ME; stderr 0.01; //measurement error shock 
    //var eps_r,eps_fx=covariance_r_fx; //interest and exchange rate shock.

end;

%%resid(1);
resid;
%%steady;
steady(solve_algo=2,maxit=20000); 
check;


%%========================================================================%
%%COMPUTATION
%=========================================================================%


stoch_simul(irf=20,order=2) Y, Yno,Yo, C, bstar, GI, GC, PX_d, RER, b, Ko, Kno, KG, p_g, N, Pstar_o, SWF, NX;


%clear all